@@ -28,7 +28,7 @@ from mypy.nodes import (MypyFile, Node, Import, ImportFrom, ImportAll,
                         SymbolTableNode, MODULE_REF)
 from mypy.semanal import FirstPass, SemanticAnalyzer, ThirdPass
 from mypy.checker import TypeChecker
-from mypy.errors import Errors, CompileError, report_internal_error
+from mypy.errors import Errors, CompileError, DecodeError, report_internal_error
 from mypy import fixup
 from mypy.report import Reports
 from mypy import defaults
@@ -66,6 +66,8 @@ DISALLOW_UNTYPED_CALLS = 'disallow-untyped-calls'
 DISALLOW_UNTYPED_DEFS = 'disallow-untyped-defs'
 # Type check unannotated functions
 CHECK_UNTYPED_DEFS = 'check-untyped-defs'
+# Also check typeshed for missing annotations
+WARN_INCOMPLETE_STUB = 'warn-incomplete-stub'
 
 PYTHON_EXTENSIONS = ['.pyi', '.py']
 
@@ -320,6 +322,7 @@ CacheMeta = NamedTuple('CacheMeta',
                         ('data_mtime', float),  # mtime of data_json
                         ('data_json', str),  # path of <id>.data.json
                         ('suppressed', List[str]),  # dependencies that weren't imported
+                        ('flags', Optional[List[str]]),  # build flags
                         ])
 # NOTE: dependencies + suppressed == all unreachable imports;
 # suppressed contains those reachable imports that were prevented by
@@ -380,7 +383,8 @@ class BuildManager:
                                         self.pyversion,
                                         DISALLOW_UNTYPED_CALLS in self.flags,
                                         DISALLOW_UNTYPED_DEFS in self.flags,
-                                        check_untyped_defs)
+                                        check_untyped_defs,
+                                        WARN_INCOMPLETE_STUB in self.flags)
         self.missing_modules = set()  # type: Set[str]
 
     def all_imported_modules_in_file(self,
@@ -521,10 +525,51 @@ find_module_cache = {}  # type: Dict[Tuple[str, Tuple[str, ...]], str]
 # in the last component.
 find_module_dir_cache = {}  # type: Dict[Tuple[str, Tuple[str, ...]], List[str]]
 
+# Cache directory listings.  We assume that while one os.listdir()
+# call may be more expensive than one os.stat() call, a small number
+# of os.stat() calls is quickly more expensive than caching the
+# os.listdir() outcome, and the advantage of the latter is that it
+# gives us the case-correct filename on Windows and Mac.
+find_module_listdir_cache = {}  # type: Dict[str, Optional[List[str]]]
+
 
 def find_module_clear_caches():
     find_module_cache.clear()
     find_module_dir_cache.clear()
+    find_module_listdir_cache.clear()
+
+
+def list_dir(path: str) -> Optional[List[str]]:
+    """Return a cached directory listing.
+
+    Returns None if the path doesn't exist or isn't a directory.
+    """
+    if path in find_module_listdir_cache:
+        return find_module_listdir_cache[path]
+    try:
+        res = os.listdir(path)
+    except OSError:
+        res = None
+    find_module_listdir_cache[path] = res
+    return res
+
+
+def is_file(path: str) -> bool:
+    """Return whether path exists and is a file.
+
+    On case-insensitive filesystems (like Mac or Windows) this returns
+    False if the case of the path's last component does not exactly
+    match the case found in the filesystem.
+    """
+    head, tail = os.path.split(path)
+    if not tail:
+        return False
+    names = list_dir(head)
+    if not names:
+        return False
+    if tail not in names:
+        return False
+    return os.path.isfile(path)
 
 
 def find_module(id: str, lib_path: Iterable[str]) -> str:
@@ -557,11 +602,15 @@ def find_module(id: str, lib_path: Iterable[str]) -> str:
         sepinit = os.sep + '__init__'
         for base_dir in candidate_base_dirs:
             base_path = base_dir + seplast  # so e.g. '/usr/lib/python3.4/foo/bar/baz'
+            # Prefer package over module, i.e. baz/__init__.py* over baz.py*.
+            for extension in PYTHON_EXTENSIONS:
+                path = base_path + sepinit + extension
+                if is_file(path) and verify_module(id, path):
+                    return path
+            # No package, look for module.
             for extension in PYTHON_EXTENSIONS:
                 path = base_path + extension
-                if not os.path.isfile(path):
-                    path = base_path + sepinit + extension
-                if os.path.isfile(path) and verify_module(id, path):
+                if is_file(path) and verify_module(id, path):
                     return path
         return None
 
@@ -637,6 +686,10 @@ def read_with_python_encoding(path: str, pyversion: Tuple[int, int]) -> str:
                 encoding = _encoding
 
         source_bytearray.extend(f.read())
+        try:
+            source_bytearray.decode(encoding)
+        except LookupError as lookuperr:
+            raise DecodeError(str(lookuperr))
         return source_bytearray.decode(encoding)
 
 
@@ -674,8 +727,7 @@ def find_cache_meta(id: str, path: str, manager: BuildManager) -> Optional[Cache
       A CacheMeta instance if the cache data was found and appears
       valid; otherwise None.
     """
-    # TODO: May need to take more build options into account; in
-    # particular SILENT_IMPORTS may affect the cache dramatically.
+    # TODO: May need to take more build options into account
     meta_json, data_json = get_cache_names(id, path, manager.pyversion)
     manager.trace('Looking for {} {}'.format(id, data_json))
     if not os.path.exists(meta_json):
@@ -696,11 +748,22 @@ def find_cache_meta(id: str, path: str, manager: BuildManager) -> Optional[Cache
         meta.get('data_mtime'),
         data_json,
         meta.get('suppressed', []),
+        meta.get('flags'),
     )
     if (m.id != id or m.path != path or
             m.mtime is None or m.size is None or
             m.dependencies is None or m.data_mtime is None):
         return None
+
+    # Metadata generated by older mypy version and no flags were saved
+    if m.flags is None:
+        return None
+
+    cached_flags = select_flags_affecting_cache(m.flags)
+    current_flags = select_flags_affecting_cache(manager.flags)
+    if cached_flags != current_flags:
+        return None
+
     # TODO: Share stat() outcome with find_module()
     st = os.stat(path)  # TODO: Errors
     if st.st_mtime != m.mtime or st.st_size != m.size:
@@ -715,6 +778,19 @@ def find_cache_meta(id: str, path: str, manager: BuildManager) -> Optional[Cache
     return m
 
 
+def select_flags_affecting_cache(flags: Iterable[str]) -> AbstractSet[str]:
+    return set(flags).intersection(FLAGS_AFFECTING_CACHE)
+
+
+FLAGS_AFFECTING_CACHE = set([
+    SILENT_IMPORTS,
+    ALMOST_SILENT,
+    DISALLOW_UNTYPED_CALLS,
+    DISALLOW_UNTYPED_DEFS,
+    CHECK_UNTYPED_DEFS,
+])
+
+
 def random_string():
     return binascii.hexlify(os.urandom(8)).decode('ascii')
 
@@ -758,14 +834,19 @@ def write_cache(id: str, path: str, tree: MypyFile,
             'data_mtime': data_mtime,
             'dependencies': dependencies,
             'suppressed': suppressed,
+            'flags': manager.flags,
             }
     with open(meta_json_tmp, 'w') as f:
         json.dump(meta, f, sort_keys=True)
         f.write('\n')
-    # TODO: On Windows, os.rename() may not be atomic, and we could
-    # use os.replace().  However that's new in Python 3.3.
-    os.rename(data_json_tmp, data_json)
-    os.rename(meta_json_tmp, meta_json)
+    # TODO: This is a temporary change until Python 3.2 support is dropped, see #1504
+    # os.rename will raise an exception rather than replace files on Windows
+    try:
+        replace = os.replace
+    except AttributeError:
+        replace = os.rename
+    replace(data_json_tmp, data_json)
+    replace(meta_json_tmp, meta_json)
 
 
 """Dependency manager.
@@ -839,7 +920,7 @@ For single nodes, processing is simple.  If the node was cached, we
 deserialize the cache data and fix up cross-references.  Otherwise, we
 do semantic analysis followed by type checking.  We also handle (c)
 above; if a module has valid cache data *but* any of its
-dependendencies was processed from source, then the module should be
+dependencies was processed from source, then the module should be
 processed from source.
 
 A relatively simple optimization (outside SCCs) we might do in the
@@ -1145,7 +1226,7 @@ class State:
                 except IOError as ioerr:
                     raise CompileError([
                         "mypy: can't read file '{}': {}".format(self.path, ioerr.strerror)])
-                except UnicodeDecodeError as decodeerr:
+                except (UnicodeDecodeError, DecodeError) as decodeerr:
                     raise CompileError([
                         "mypy: can't decode file '{}': {}".format(self.path, str(decodeerr))])
             self.tree = manager.parse_file(self.id, self.xpath, source)
@@ -1376,7 +1457,7 @@ def process_graph(graph: Graph, manager: BuildManager) -> None:
         else:
             fresh_msg = "stale due to deps (%s)" % " ".join(sorted(stale_deps))
         if len(scc) == 1:
-            manager.log("Processing SCC sigleton (%s) as %s" % (" ".join(scc), fresh_msg))
+            manager.log("Processing SCC singleton (%s) as %s" % (" ".join(scc), fresh_msg))
         else:
             manager.log("Processing SCC of size %d (%s) as %s" %
                         (len(scc), " ".join(scc), fresh_msg))
