@@ -4,7 +4,8 @@ from typing import cast, Callable, List, Optional
 
 from mypy.types import (
     Type, Instance, AnyType, TupleType, CallableType, FunctionLike, TypeVarDef,
-    Overloaded, TypeVarType, TypeTranslator, UnionType, PartialType, DeletedType, NoneTyp
+    Overloaded, TypeVarType, TypeTranslator, UnionType, PartialType,
+    DeletedType, NoneTyp, TypeType
 )
 from mypy.nodes import TypeInfo, FuncBase, Var, FuncDef, SymbolNode, Context
 from mypy.nodes import ARG_POS, ARG_STAR, ARG_STAR2, function_type, Decorator, OverloadedFuncDef
@@ -113,6 +114,23 @@ def analyze_member_access(name: str, typ: Type, node: Context, is_lvalue: bool,
     elif isinstance(typ, DeletedType):
         msg.deleted_as_rvalue(typ, node)
         return AnyType()
+    elif isinstance(typ, TypeType):
+        # Similar to FunctionLike + is_type_obj() above.
+        item = None
+        if isinstance(typ.item, Instance):
+            item = typ.item
+        elif isinstance(typ.item, TypeVarType):
+            if isinstance(typ.item.upper_bound, Instance):
+                item = typ.item.upper_bound
+        if item:
+            result = analyze_class_attribute_access(item, name, node, is_lvalue,
+                                                    builtin_type, not_ready_callback, msg)
+            if result:
+                return result
+        fallback = builtin_type('builtins.type')
+        return analyze_member_access(name, fallback, node, is_lvalue, is_super,
+                                     builtin_type, not_ready_callback, msg,
+                                     report_type=report_type)
     return msg.has_no_attr(report_type, name, node)
 
 
@@ -237,6 +255,10 @@ def check_method_type(functype: FunctionLike, itype: Instance, is_classmethod: b
         elif not is_classmethod:
             # Check that self argument has type 'Any' or valid instance type.
             selfarg = item.arg_types[0]
+            # If this is a method of a tuple class, correct for the fact that
+            # we passed to typ.fallback in analyze_member_access. See #1432.
+            if isinstance(selfarg, TupleType):
+                selfarg = selfarg.fallback
             if not subtypes.is_equivalent(selfarg, itype):
                 msg.invalid_method_type(item, context)
         else:
@@ -463,7 +485,7 @@ def map_type_from_supertype(typ: Type, sub_info: TypeInfo,
     inst_type = map_instance_to_supertype(inst_type, super_info)
     # Finally expand the type variables in type with those in the previously
     # constructed type. Note that both type and inst_type may have type
-    # variables, but in type they are interpreterd in supertype context while
+    # variables, but in type they are interpreted in supertype context while
     # in inst_type they are interpreted in subtype context. This works even if
     # the names of type variables in supertype and subtype overlap.
     return expand_type_by_instance(typ, inst_type)
