@@ -38,6 +38,9 @@ except ImportError:
 T = TypeVar('T')
 U = TypeVar('U')
 
+TYPE_COMMENT_SYNTAX_ERROR = 'syntax error in type comment'
+TYPE_COMMENT_AST_ERROR = 'invalid type comment'
+
 
 def parse(source: Union[str, bytes], fnam: str = None, errors: Errors = None,
           pyversion: Tuple[int, int] = defaults.PYTHON3_VERSION,
@@ -56,19 +59,20 @@ def parse(source: Union[str, bytes], fnam: str = None, errors: Errors = None,
         else:
             ast2 = ast27.parse(source, fnam, 'exec')
             ast = conversions.py2to3(ast2)
-    except SyntaxError as e:
-        if errors:
-            errors.set_file('<input>' if fnam is None else fnam)
-            errors.report(e.lineno, e.msg)  # type: ignore
-        else:
-            raise
-    else:
+
         tree = ASTConverter(pyversion=pyversion,
+                            is_stub=is_stub_file,
                             custom_typing_module=custom_typing_module,
                             ).visit(ast)
         tree.path = fnam
         tree.is_stub = is_stub_file
         return tree
+    except (SyntaxError, TypeCommentParseError) as e:
+        if errors:
+            errors.set_file('<input>' if fnam is None else fnam)
+            errors.report(e.lineno, e.msg)
+        else:
+            raise
 
     return MypyFile([],
                     [],
@@ -78,9 +82,13 @@ def parse(source: Union[str, bytes], fnam: str = None, errors: Errors = None,
 
 
 def parse_type_comment(type_comment: str, line: int) -> Type:
-    typ = ast35.parse(type_comment, '<type_comment>', 'eval')
-    assert isinstance(typ, ast35.Expression)
-    return TypeConverter(line=line).visit(typ.body)
+    try:
+        typ = ast35.parse(type_comment, '<type_comment>', 'eval')
+    except SyntaxError:
+        raise TypeCommentParseError(TYPE_COMMENT_SYNTAX_ERROR, line)
+    else:
+        assert isinstance(typ, ast35.Expression)
+        return TypeConverter(line=line).visit(typ.body)
 
 
 def with_line(f: Callable[[Any, T], U]) -> Callable[[Any, T], U]:
@@ -100,11 +108,15 @@ def find(f: Callable[[T], bool], seq: Sequence[T]) -> T:
 
 
 class ASTConverter(ast35.NodeTransformer):
-    def __init__(self, pyversion: Tuple[int, int], custom_typing_module: str = None) -> None:
+    def __init__(self,
+                 pyversion: Tuple[int, int],
+                 is_stub: bool,
+                 custom_typing_module: str = None) -> None:
         self.class_nesting = 0
         self.imports = []  # type: List[ImportBase]
 
         self.pyversion = pyversion
+        self.is_stub = is_stub
         self.custom_typing_module = custom_typing_module
 
     def generic_visit(self, node: ast35.AST) -> None:
@@ -234,7 +246,10 @@ class ASTConverter(ast35.NodeTransformer):
         arg_names = [arg.variable.name() for arg in args]
         arg_types = None  # type: List[Type]
         if n.type_comment is not None:
-            func_type_ast = ast35.parse(n.type_comment, '<func_type>', 'func_type')
+            try:
+                func_type_ast = ast35.parse(n.type_comment, '<func_type>', 'func_type')
+            except SyntaxError:
+                raise TypeCommentParseError(TYPE_COMMENT_SYNTAX_ERROR, n.lineno)
             assert isinstance(func_type_ast, ast35.FunctionType)
             # for ellipsis arg
             if (len(func_type_ast.argtypes) == 1 and
@@ -644,16 +659,31 @@ class ASTConverter(ast35.NodeTransformer):
 
         raise RuntimeError('num not implemented for ' + str(type(n.n)))
 
-    # Str(string s) -- need to specify raw, unicode, etc?
+    # Str(string s)
     @with_line
     def visit_Str(self, n: ast35.Str) -> Node:
-        return StrExpr(n.s)
+        if self.pyversion[0] >= 3 or self.is_stub:
+            # Hack: assume all string literals in Python 2 stubs are normal
+            # strs (i.e. not unicode).  All stubs are parsed with the Python 3
+            # parser, which causes unprefixed string literals to be interpreted
+            # as unicode instead of bytes.  This hack is generally okay,
+            # because mypy considers str literals to be compatible with
+            # unicode.
+            return StrExpr(n.s)
+        else:
+            return UnicodeExpr(n.s)
 
     # Bytes(bytes s)
     @with_line
     def visit_Bytes(self, n: ast35.Bytes) -> Node:
-        # TODO: this is kind of hacky
-        return BytesExpr(str(n.s)[2:-1])
+        # The following line is a bit hacky, but is the best way to maintain
+        # compatibility with how mypy currently parses the contents of bytes literals.
+        contents = str(n.s)[2:-1]
+
+        if self.pyversion[0] >= 3:
+            return BytesExpr(contents)
+        else:
+            return StrExpr(contents)
 
     # NameConstant(singleton value)
     def visit_NameConstant(self, n: ast35.NameConstant) -> Node:
@@ -721,7 +751,7 @@ class TypeConverter(ast35.NodeTransformer):
         self.line = line
 
     def generic_visit(self, node: ast35.AST) -> None:
-        raise RuntimeError('Type node not implemented: ' + str(type(node)))
+        raise TypeCommentParseError(TYPE_COMMENT_AST_ERROR, self.line)
 
     def visit_NoneType(self, n: Any) -> Type:
         return None
@@ -774,3 +804,9 @@ class TypeConverter(ast35.NodeTransformer):
     # List(expr* elts, expr_context ctx)
     def visit_List(self, n: ast35.List) -> Type:
         return TypeList(self.visit_list(n.elts), line=self.line)
+
+
+class TypeCommentParseError(Exception):
+    def __init__(self, msg: str, lineno: int) -> None:
+        self.msg = msg
+        self.lineno = lineno
