@@ -3,7 +3,8 @@ from typing import cast, List
 from mypy.join import is_similar_callables, combine_similar_callables
 from mypy.types import (
     Type, AnyType, TypeVisitor, UnboundType, Void, ErrorType, NoneTyp, TypeVarType,
-    Instance, CallableType, TupleType, ErasedType, TypeList, UnionType, PartialType, DeletedType
+    Instance, CallableType, TupleType, ErasedType, TypeList,
+    UnionType, PartialType, DeletedType, TypeType
 )
 from mypy.subtypes import is_subtype
 from mypy.nodes import TypeInfo
@@ -43,7 +44,7 @@ def is_overlapping_types(t: Type, s: Type, use_promotions: bool = False) -> bool
     variables are erased at runtime and the overlapping check is based
     on runtime behavior.
 
-    If use_promitions is True, also consider type promotions (int and
+    If use_promotions is True, also consider type promotions (int and
     float would only be overlapping if it's True).
 
     This does not consider multiple inheritance. For example, A and B in
@@ -55,7 +56,7 @@ def is_overlapping_types(t: Type, s: Type, use_promotions: bool = False) -> bool
         class C(A, B): ...
 
     The rationale is that this case is usually very unlikely as multiple
-    inhreitance is rare. Also, we can't reliably determine whether
+    inheritance is rare. Also, we can't reliably determine whether
     multiple inheritance actually occurs somewhere in a program, due to
     stub files hiding implementation details, dynamic loading etc.
 
@@ -184,6 +185,8 @@ class TypeMeetVisitor(TypeVisitor[Type]):
                     return self.s
                 else:
                     return NoneTyp()
+        elif isinstance(self.s, TypeType):
+            return meet_types(t, self.s)
         else:
             return self.default(self.s)
 
@@ -207,6 +210,17 @@ class TypeMeetVisitor(TypeVisitor[Type]):
         # We can't determine the meet of partial types. We should never get here.
         assert False, 'Internal error'
 
+    def visit_type_type(self, t: TypeType) -> Type:
+        if isinstance(self.s, TypeType):
+            typ = self.meet(t.item, self.s.item)
+            if not isinstance(typ, NoneTyp):
+                typ = TypeType(typ, line=t.line)
+            return typ
+        elif isinstance(self.s, Instance) and self.s.type.fullname() == 'builtins.type':
+            return t
+        else:
+            return self.default(self.s)
+
     def meet(self, s, t):
         return meet_types(s, t)
 
