@@ -11,7 +11,7 @@ from typing import cast, List, Dict, Any, Sequence, Iterable, Tuple
 from mypy.errors import Errors
 from mypy.types import (
     Type, CallableType, Instance, TypeVarType, TupleType, UnionType, Void, NoneTyp, AnyType,
-    Overloaded, FunctionLike, DeletedType
+    Overloaded, FunctionLike, DeletedType, TypeType
 )
 from mypy.nodes import (
     TypeInfo, Context, MypyFile, op_methods, FuncDef, reverse_type_aliases,
@@ -81,6 +81,7 @@ RETURN_TYPE_EXPECTED = "Function is missing a return type annotation"
 ARGUMENT_TYPE_EXPECTED = "Function is missing a type annotation for one or more arguments"
 KEYWORD_ARGUMENT_REQUIRES_STR_KEY_TYPE = \
     'Keyword argument only valid with "str" key type in call to "dict"'
+ALL_MUST_BE_SEQ_STR = 'Type of __all__ must be {}, not {}'
 
 
 class MessageBuilder:
@@ -140,7 +141,8 @@ class MessageBuilder:
     def report(self, msg: str, context: Context, severity: str, file: str = None) -> None:
         """Report an error or note (unless disabled)."""
         if self.disable_count <= 0:
-            self.errors.report(context.get_line(), msg.strip(), severity=severity, file=file)
+            self.errors.report(context.get_line() if context else -1,
+                               msg.strip(), severity=severity, file=file)
 
     def fail(self, msg: str, context: Context, file: str = None) -> None:
         """Report an error message (unless disabled)."""
@@ -263,6 +265,9 @@ class MessageBuilder:
             return '"Any"'
         elif isinstance(typ, DeletedType):
             return '<deleted>'
+        elif isinstance(typ, TypeType):
+            return 'Type[{}]'.format(
+                strip_quotes(self.format_simple(typ.item, verbosity)))
         elif typ is None:
             raise RuntimeError('Type is None')
         else:
@@ -288,7 +293,7 @@ class MessageBuilder:
     # Specific operations
     #
 
-    # The following operations are for genering specific error messages. They
+    # The following operations are for generating specific error messages. They
     # get some information as arguments, and they build an error message based
     # on them.
 
@@ -437,7 +442,12 @@ class MessageBuilder:
                 if n == 1:
                     self.invalid_index_type(arg_type, base, context)
                 else:
-                    self.fail(INCOMPATIBLE_TYPES_IN_ASSIGNMENT, context)
+                    msg = '{} (expression has type {}, target has type {})'
+                    arg_type_str, callee_type_str = self.format_distinctly(arg_type,
+                                                                           callee.arg_types[n - 1])
+                    self.fail(msg.format(INCOMPATIBLE_TYPES_IN_ASSIGNMENT,
+                                         arg_type_str, callee_type_str),
+                              context)
                 return
 
             target = 'to {} '.format(name)
@@ -818,6 +828,12 @@ class MessageBuilder:
     def invalid_signature(self, func_type: Type, context: Context) -> None:
         self.fail('Invalid signature "{}"'.format(func_type), context)
 
+    def reveal_type(self, typ: Type, context: Context) -> None:
+        self.fail('Revealed type is \'{}\''.format(typ), context)
+
+    def unsupported_type_type(self, item: Type, context: Context) -> None:
+        self.fail('Unsupported type Type[{}]'.format(self.format(item)), context)
+
 
 def capitalize(s: str) -> str:
     """Capitalize the first character of a string."""
