@@ -308,7 +308,7 @@ class OverloadedFuncDef(FuncBase):
         self.set_line(items[0].line)
 
     def name(self) -> str:
-        return self.items[1].func.name()
+        return self.items[0].func.name()
 
     def accept(self, visitor: NodeVisitor[T]) -> T:
         return visitor.visit_overloaded_func_def(self)
@@ -719,7 +719,7 @@ class Block(Node):
 
 
 class ExpressionStmt(Node):
-    """An expression as a statament, such as print(s)."""
+    """An expression as a statement, such as print(s)."""
     expr = None  # type: Node
 
     def __init__(self, expr: Node) -> None:
@@ -961,6 +961,17 @@ class IntExpr(Node):
         return visitor.visit_int_expr(self)
 
 
+# How mypy uses StrExpr, BytesExpr, and UnicodeExpr:
+# In Python 2 mode:
+# b'x', 'x' -> StrExpr
+# u'x' -> UnicodeExpr
+# BytesExpr is unused
+#
+# In Python 3 mode:
+# b'x' -> BytesExpr
+# 'x', u'x' -> StrExpr
+# UnicodeExpr is unused
+
 class StrExpr(Node):
     """String literal"""
 
@@ -1368,6 +1379,18 @@ class CastExpr(Node):
         return visitor.visit_cast_expr(self)
 
 
+class RevealTypeExpr(Node):
+    """Reveal type expression reveal_type(expr)."""
+
+    expr = None  # type: Node
+
+    def __init__(self, expr: Node) -> None:
+        self.expr = expr
+
+    def accept(self, visitor: NodeVisitor[T]) -> T:
+        return visitor.visit_reveal_type_expr(self)
+
+
 class SuperExpr(Node):
     """Expression super().name"""
 
@@ -1713,6 +1736,8 @@ class TypeInfo(SymbolNode):
     names = None  # type: SymbolTable      # Names defined directly in this type
     is_abstract = False                    # Does the class have any abstract attributes?
     abstract_attributes = None  # type: List[str]
+    # Classes inheriting from Enum shadow their true members with a __getattr__, so we
+    # have to treat them as a special case.
     is_enum = False
     # If true, any unknown attributes should have type 'Any' instead
     # of generating a type error.  This would be true if there is a
